from functools import wraps

from django.http import HttpResponse

def file_response(func):
    @wraps(func)
    def provider_response(*args, **kwargs):
        result = func(*args, **kwargs)
        if isinstance(result, basestring):
           response = HttpResponse()
           response['X-File'] = result
        return response
    return provider_response


def file_response_cached(func):
    @wraps(func)
    def provider_response(*args, **kwargs):
        result = func(*args, **kwargs)
        if isinstance(result, basestring):
           response = HttpResponse()
           response['X-File'] = result
        return response
    return provider_response




import os
import six

from django.conf import settings
from django.core.files.base import File
from django.http import HttpResponse, HttpResponseNotFound, FileResponse

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object

class FileProvider(object):
      def get_response(self, response, **options):
          return self._get_response(response, **options)

      def _get_response(self, response, **options):
          raise NotImplemented

class XAccelFileProvider(FileProvider):
      def _get_response(self, response, **options):
          response['X-Accel-Redirect'] = response['X-File']
          del response['X-File']
          return response

class XSendFileProvider(FileProvider):
      def _get_response(self, response, **options):
          response['X-Sendfile'] = response['X-File']
          del response['X-File']
          return response

class PythonFileProvider(FileProvider):
      def _get_response(self, response, **options):
          # only need this checking when python file provider is given
          if not os.path.exists(response['X-File']):
              return HttpResponseNotFound("file not found")
          return FileResponse(open(response['X-File'], 'rb'))

# Uses X-Sendfile
ApacheFileProvider = XSendFileProvider
LightHttpdFileProvider = XSendFileProvider
HiawathaFileProvider = XSendFileProvider
# Uses X-Accel-Redirect
NginxFileProvider = XAccelFileProvider
CaddyFileProvider = XAccelFileProvider

PROVIDERS = {
 'python': PythonFileProvider,
 'nginx': NginxFileProvider,
 'apache': ApacheFileProvider,
 'lighthttpd': LightHttpdFileProvider,
 'caddy': CaddyFileProvider,
 'hiawatha': HiawathaFileProvider,
 'xaccel': XAccelFileProvider,
 'xsendfile': XSendFileProvider,
}

class FileProviderMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        filepath = response.get('X-File', "")
        if filepath != "" and isinstance(filepath, six.string_types):
            provider_name = getattr(settings, "FILEPROVIDER_NAME", "python")
            # provider_option = getattr(settings, "DJM_ENABLE_CACHE", True)
            provider = PROVIDERS[provider_name]
            response =  provider().get_response(response)
        return response



from setuptools import setup

__version__  = '0.1.3'
 
setup(
    name='django-fileprovider',
    version=__version__,
    description='django middleware for serving media files ( or protect them )',
    author='Renjith Thankachan',
    author_email='mail3renjith@gmail.com',
    url='https://github.com/instapk/django-fileprovider.git',
    download_url='https://github.com/instapk/django-fileprovider/archive/0.1.3.tar.gz',
    long_description=open('README.md', 'r').read(),
    packages=[
        'fileprovider',
    ],
    zip_safe=False,
    requires=[
    ],
    install_requires=[
      'six>=1.10.0',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Utilities'
    ],
)
